
`timescale 1 ns/100 ps

`include "parameters.v"

module TB_CAM_RAMB16();

reg [8:0] DATA_IN;
reg [4:0] ADDR;
reg WRITE_ENABLE;
reg ERASE_WRITE;
reg WRITE_RAM;
reg CLK;
reg MATCH_ENABLE;
reg MATCH_RST;
wire [31:0] MATCH;


// CAM_RAMB16 instantiation

CAM_RAMB16 UUT (.DATA_IN(DATA_IN), .ADDR(ADDR), .WRITE_ENABLE(WRITE_ENABLE),
               .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK),
               .MATCH_ENABLE(MATCH_ENABLE), .MATCH_RST(MATCH_RST), .MATCH(MATCH));
               
// Clock Generation

initial
  CLK = 1'b0;
               
always
  #(`clock_period/2) CLK = ~CLK;
  
  
// Defining Stimulus

initial
 begin
 
  DATA_IN = 9'h000;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'hXXXXXXXX;    // Expected value of the output
  
  delay_1_clock;
  
  // Synchronous Reset
  DATA_IN = 9'h000;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b0;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  // 1-Write Operation DATA_IN at the address ADDR
  DATA_IN = 9'h001;
  ADDR = 5'h02;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b1;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h000;
  ADDR = 5'h02;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b1;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  // Dummy clock cycle
  DATA_IN = 9'h000;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  // 1-Read Operation of the first data
  DATA_IN = 9'h001;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000004;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h000;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000004;    // Expected value of the output
  
  delay_1_clock;
  
  // 2-Write Operation DATA_IN at the address ADDR
  DATA_IN = 9'h0DF;
  ADDR = 5'h0C;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b1;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h0DF;
  ADDR = 5'h0C;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b1;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  // 3-Write Operation DATA_IN at the address ADDR
  DATA_IN = 9'h0A0;
  ADDR = 5'h0F;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b1;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h0A0;
  ADDR = 5'h0F;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b1;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  // 4-Write Operation DATA_IN at the address ADDR
  DATA_IN = 9'h069;
  ADDR = 5'h05;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b1;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h000;
  ADDR = 5'h05;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b1;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  // 4 & 3 & 2 & non-existing data & 1 read back
  DATA_IN = 9'h069;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000020;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h0A0;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00008000;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h0DF;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00001000;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h061;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h001;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000004;    // Expected value of the output
  
  delay_1_clock;
  
  // 5-Write Operation DATA_IN at the address ADDR (same as the first address)
  DATA_IN = 9'h0FF;
  ADDR = 5'h02;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b1;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000004;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h0FF;
  ADDR = 5'h02;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b1;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000004;    // Expected value of the output
  
  delay_1_clock;
  
  // 1 read back then erase n 1
  DATA_IN = 9'h001;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  // 6-Write Operation DATA_IN (same as n 1) at the address ADDR
  DATA_IN = 9'h001;
  ADDR = 5'h03;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b1;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h001;
  ADDR = 5'h03;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b1;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  // 5 & 6 & non-existing data read back
  DATA_IN = 9'h0FF;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000004;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h081;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h001;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000008;    // Expected value of the output
  
  delay_1_clock;
  
  // Dummy clock cycle/Test Match Reset
  DATA_IN = 9'h000;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b0;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  // 7-Write Operation DATA_IN at the address ADDR "0000"
  DATA_IN = 9'h001;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b1;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h001;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b1;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  // 8-Write Operation DATA_IN (same as n 4) at the address ADDR
  DATA_IN = 9'h0C2;
  ADDR = 5'h05;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b1;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h0C2;
  ADDR = 5'h05;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b1;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  // Multi-matched ( n 6 & n 7) & 8 & 5 read back
  DATA_IN = 9'h001;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000009;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h0C2;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000020;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h0FF;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000004;    // Expected value of the output
  
  delay_1_clock;
  
  // Dummy Clock cycles
  DATA_IN = 9'h000;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000004;    // Expected value of the output
  
  delay_1_clock;
  
  //9-Write Operation DATA_IN (same as n4 & n8) at the address ADDR
  DATA_IN = 9'h0C2;
  ADDR = 5'h0C;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b1;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000004;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h0C2;
  ADDR = 5'h0C;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b1;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000004;    // Expected value of the output
  
  delay_1_clock;
  
  // 10-Write Operation DATA_IN (same as n4 & n8) at the address ADDR 
  DATA_IN = 9'h0C2;
  ADDR = 5'h07;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b1;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000004;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h0C2;
  ADDR = 5'h07;
  WRITE_ENABLE = 1'b1;
  ERASE_WRITE = 1'b1;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000004;    // Expected value of the output
  
  delay_1_clock;
  
  // Read Multi-matches (n 8 & n 9 & n 10) read back
  DATA_IN = 9'h001;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000009;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h0C2;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b1;
  //MATCH = 32'h000010A0;    // Expected value of the output
  
  delay_1_clock;
  
  // Synchronous Reset
  DATA_IN = 9'h000;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b1;
  MATCH_RST = 1'b0;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  // Dummy Clcok cycle
  DATA_IN = 9'h000;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  delay_1_clock;
  
  DATA_IN = 9'h000;
  ADDR = 5'h00;
  WRITE_ENABLE = 1'b0;
  ERASE_WRITE = 1'b0;
  WRITE_RAM = 1'b0;
  MATCH_ENABLE = 1'b0;
  MATCH_RST = 1'b1;
  //MATCH = 32'h00000000;    // Expected value of the output
  
  end // initial for Defining Stimulus
  
  task delay_1_clock;
     begin
       #1;
       @ (posedge CLK);
       #6;
     end
  endtask  // delay_1_clock
  
  endmodule // TB_CAM_RAMB16
  
  
  
  
  
  
  
  
  
  
  
    
  
  
  
  
  
  
   
  
  
  

               





