--
-- Module: 	DECODE_2
-- Design: 	CAM_Top
-- VHDL code:	 RTL / Combinatorial
--
-- Synthesis	Synopsys FPGA Express ver. 3.6 
--		Use of "pragma synthesis_off/on" and attributes
--
-- Description: Decode 2 bits address into 4 binary bits
--		Generate an ENABLE bus
--
-- Device: 	VIRTEX -II Family 
--
-- Created by: Jean-Louis BRELET & Lakshmi Gopalakrishnan / XILINX - VIRTEX-II  Applications
-- Date: July 23, 2001
-- Version: 1.0
--
-- 
--
--
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--  Copyright (c) 1999 Xilinx, Inc.  All rights reserved.
-------------------------------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

-- Syntax for Synopsys FPGA Express
-- pragma translate_off
--library UNISIM;
--use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on

entity DECODE_2 is
    port (
        ADDR		: in std_logic_vector (1 downto 0);
        ENABLE		: in std_logic;
        BINARY_ADDR	: out std_logic_vector (3 downto 0)
    );
end DECODE_2;

architecture DECODE_2_arch of DECODE_2 is
--
-- Components Declarations:
--
-- signal VCC : std_logic;
-- signal GND : std_logic;
--
begin
-- VCC <= '1';
-- GND <= '0';
--
-- Create the write enable signal for each CAM_RAMB16
DECODE: process (ADDR,ENABLE)
begin
	BINARY_ADDR <= ( others => '0');
	case ADDR(1 downto 0) is
		when "00" => BINARY_ADDR(0) <= ENABLE;
		when "01" => BINARY_ADDR(1) <= ENABLE;
		when "10" => BINARY_ADDR(2) <= ENABLE;
		when "11" => BINARY_ADDR(3) <= ENABLE;
		when others => BINARY_ADDR <= ( others => 'X');
	end case;
end process DECODE;	
--									
end DECODE_2_arch;
