--
-- Module: 	DECODE_4
-- Design: 	CAM_Top
-- VHDL code:	 RTL / Combinatorial
--
-- Synthesis	Synopsys FPGA Express ver. 3.6 
--		Use of "pragma synthesis_off/on" and attributes
--
-- Description: Decode 4 bits address into 16 binary bits
--		Generate an ENABLE bus
--
-- Device: 	VIRTEX-II  Family 
--
-- Created by: Jean-Louis BRELET & Lakshmi Gopalakrishnan / XILINX - VIRTEX-II Applications
-- Date: July 23, 2001
-- Version: 1.0
--
-- 
--
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--  Copyright (c) 1999 Xilinx, Inc.  All rights reserved.
-------------------------------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

-- Syntax for Synopsys FPGA Express
-- pragma translate_off
--library UNISIM;
--use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on

entity DECODE_4 is
    port (
        ADDR		: in std_logic_vector (3 downto 0);
        ENABLE		: in std_logic;
        BINARY_ADDR	: out std_logic_vector (15 downto 0)
    );
end DECODE_4;

architecture DECODE_4_arch of DECODE_4 is
--
-- Components Declarations:
--
-- signal VCC : std_logic;
-- signal GND : std_logic;
--
begin
-- VCC <= '1';
-- GND <= '0';
--
-- Create the write enable signal for each CAM_RAMB16
DECODE: process (ADDR,ENABLE)
begin
	BINARY_ADDR <= ( others => '0');
	case ADDR(3 downto 0) is
		when "0000" => BINARY_ADDR(0) <= ENABLE;
		when "0001" => BINARY_ADDR(1) <= ENABLE;
		when "0010" => BINARY_ADDR(2) <= ENABLE;
		when "0011" => BINARY_ADDR(3) <= ENABLE;
		when "0100" => BINARY_ADDR(4) <= ENABLE;
		when "0101" => BINARY_ADDR(5) <= ENABLE;
		when "0110" => BINARY_ADDR(6) <= ENABLE;
		when "0111" => BINARY_ADDR(7) <= ENABLE;	
		when "1000" => BINARY_ADDR(8) <= ENABLE;
		when "1001" => BINARY_ADDR(9) <= ENABLE;
		when "1010" => BINARY_ADDR(10) <= ENABLE;
		when "1011" => BINARY_ADDR(11) <= ENABLE;
		when "1100" => BINARY_ADDR(12) <= ENABLE;
		when "1101" => BINARY_ADDR(13) <= ENABLE;
		when "1110" => BINARY_ADDR(14) <= ENABLE;
		when "1111" => BINARY_ADDR(15) <= ENABLE;					
		when others => BINARY_ADDR <= ( others => 'X');
	end case;
end process DECODE;	
--									
end DECODE_4_arch;
