--
-- Module: 	ENCODE_5_LSB
-- Design: 	CAM_Top
-- VHDL code:	RTL / Combinatorial
--
-- Synthesis	Synopsys FPGA Express ver. 3.6 
--		Use of "pragma synthesis_off/on" and attributes
--
-- Description: Encode a 32 bits binary address into 5 bits and find if a match occurs
--              if BINARY_ADDR = "00000000000000000000010000000000" => MATCH_ADDR = "01010"/ MATCH_OK = 1 
--		Optional ADDR_VALID = 1 when only one Match (If simultaneous matches can occur)
--		However, the ADDR_VALID generation double the size of the combinatorial logic !
--		if no match found => MATCH_OK = 0 / ADDR_VALID = 0 (MATCH_ADDR is not a valid address)
--		if 2 or more matches found => MATCH_OK = 1 / ADDR_VALID = 0 (MATCH_ADDR is not valid address)
--
-- Device: 	VIRTEX - II  Family 
--
-- Created by: Jean-Louis BRELET & Lakshmi Gopalakrishnan / XILINX - VIRTEX -II Applications
-- Date: July 23, 2001
-- Version: 1.0
--
-- 
--
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--  Copyright (c) 1999 Xilinx, Inc.  All rights reserved.
-------------------------------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

-- Syntax for Synopsys FPGA Express
-- pragma translate_off
--library UNISIM;
--use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on

entity ENCODE_5_LSB is
    port (
        BINARY_ADDR	: in std_logic_vector(31 downto 0);
        MATCH_ADDR	: out std_logic_vector(4 downto 0); -- Match address found
--       ADDR_VALID	: out std_logic; -- '1' if MATCH_ADDR is valid (Only one match)
        MATCH_OK		: out std_logic -- '1' if MATCH found
    );
end ENCODE_5_LSB;

architecture ENCODE_5_LSB_arch of ENCODE_5_LSB is
--
-- Components Declarations:
--
-- Signal Declarations:
-- signal VCC : std_logic;
-- signal GND : std_logic;
--
begin
-- VCC <= '1';
-- GND <= '0';

-- Convert the binary address in an address bus, ONLY is ONE match is found
-- ADDR_VALID signal generation double the logic size of the encoder !!!
GENERATE_ADDRESS : process (BINARY_ADDR)
begin
--	ADDR_VALID <= '1';
	case BINARY_ADDR(31 downto 0) is	
		when "00000000000000000000000000000001" => MATCH_ADDR <= "00000";
		when "00000000000000000000000000000010" => MATCH_ADDR <= "00001";
		when "00000000000000000000000000000100" => MATCH_ADDR <= "00010";
		when "00000000000000000000000000001000" => MATCH_ADDR <= "00011";
		when "00000000000000000000000000010000" => MATCH_ADDR <= "00100";
		when "00000000000000000000000000100000" => MATCH_ADDR <= "00101";
		when "00000000000000000000000001000000" => MATCH_ADDR <= "00110";
		when "00000000000000000000000010000000" => MATCH_ADDR <= "00111";
		when "00000000000000000000000100000000" => MATCH_ADDR <= "01000";
		when "00000000000000000000001000000000" => MATCH_ADDR <= "01001";
		when "00000000000000000000010000000000" => MATCH_ADDR <= "01010";
		when "00000000000000000000100000000000" => MATCH_ADDR <= "01011";
		when "00000000000000000001000000000000" => MATCH_ADDR <= "01100";
		when "00000000000000000010000000000000" => MATCH_ADDR <= "01101";
		when "00000000000000000100000000000000" => MATCH_ADDR <= "01110";
		when "00000000000000001000000000000000" => MATCH_ADDR <= "01111";
                when "00000000000000010000000000000000" => MATCH_ADDR <= "10000";
                when "00000000000000100000000000000000" => MATCH_ADDR <= "10001";
                when "00000000000001000000000000000000" => MATCH_ADDR <= "10010";
                when "00000000000010000000000000000000" => MATCH_ADDR <= "10011";
                when "00000000000100000000000000000000" => MATCH_ADDR <= "10100";
                when "00000000001000000000000000000000" => MATCH_ADDR <= "10101";
                when "00000000010000000000000000000000" => MATCH_ADDR <= "10110"; 
                when "00000000100000000000000000000000" => MATCH_ADDR <= "10111";
                when "00000001000000000000000000000000" => MATCH_ADDR <= "11000";
                when "00000010000000000000000000000000" => MATCH_ADDR <= "11001";
                when "00000100000000000000000000000000" => MATCH_ADDR <= "11010";
                when "00001000000000000000000000000000" => MATCH_ADDR <= "11011";
                when "00010000000000000000000000000000" => MATCH_ADDR <= "11100";
                when "00100000000000000000000000000000" => MATCH_ADDR <= "11101";
                when "01000000000000000000000000000000" => MATCH_ADDR <= "11110";
                when "10000000000000000000000000000000" => MATCH_ADDR <= "11111";
		when others => 
			MATCH_ADDR <= ( others => 'X');										
--			ADDR_VALID <= '0';
	end case;
end process GENERATE_ADDRESS;

-- Generate the match signal if one or more matche(s) is/are found
GENERATE_MATCH : process (BINARY_ADDR)
begin
if (BINARY_ADDR = "00000000000000000000000000000000") then
	MATCH_OK <= '0';
else
	MATCH_OK <= '1';
end if;
end process GENERATE_MATCH;

end ENCODE_5_LSB_arch;
