--
-- Module: 	INIT_9_RAM32x1s
-- Design: 	CAM_Top
-- VHDL code:	VIRTEX primitives instantiation
--
-- Synthesis	Synopsys FPGA Express ver. 3.6 
--		Use of "pragma synthesis_off/on" and attributes
--
-- Description: Basic building block of a CAM using Select BlockRAM & Select RAM
--		Instantiate 9 RAM32x1s_1 for ERASE operation
--		Initialization of RAM32x1s: attributes to constraint PAR and simulation
--
-- Device: 	VIRTEX-II Family 
--		9 x SelectRAM32x1s_1
--
-- Created by: Jean-Louis BRELET & Lakshmi Gopalakrishnan / XILINX - VIRTEX-II Applications
-- Date: July 23, 2001
-- Version: 1.0
--
--
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--  Copyright (c) 1999 Xilinx, Inc.  All rights reserved.
-------------------------------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

-- Syntax for Synopsys FPGA Express
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on


entity INIT_9_RAM32x1s is
    port (
	DATA_IN	: in std_logic_vector(8 downto 0) ; 	
	ADDR		: in std_logic_vector(4 downto 0) ;	-- Used by erase/write operation only
	WRITE_RAM	: in std_logic;		-- if '1' DATA_IN is WRITE in the RAM32x1s
	CLK		: in std_logic;
	DATA_WRITE	: out std_logic_vector(8 downto 0) 
	);
end INIT_9_RAM32x1s;

architecture INIT_9_RAM32x1s_arch of INIT_9_RAM32x1s is
--
-- Components Declarations:
-- RAM32x9s macro is not used because the INIT attribute can be specified at the primitive level only.
component RAM32x1s_1
-- RAM initialization for RTL simulation 
-- Syntax for Synopsys FPGA Express
-- pragma synthesis_off
generic( 
      INIT : bit_vector(31 downto 0) := X"00000000"
  );
-- pragma synthesis_on	
    port (
	WE	: in std_logic;
	WCLK	: in std_logic; -- inverted Clock
	D	: in std_logic;
	A0	: in std_logic;
	A1	: in std_logic;
	A2	: in std_logic;
	A3	: in std_logic;
	A4 : in std_logic;
	O	: out std_logic
    );
end component;
	
--
-- RAM32x9s memories initialization for Alliance
attribute INIT: string;
attribute INIT of RAM_ERASE_0: label is "00000000";
attribute INIT of RAM_ERASE_1: label is "00000000";
attribute INIT of RAM_ERASE_2: label is "00000000";
attribute INIT of RAM_ERASE_3: label is "00000000";
attribute INIT of RAM_ERASE_4: label is "00000000";
attribute INIT of RAM_ERASE_5: label is "00000000";
attribute INIT of RAM_ERASE_6: label is "00000000";
attribute INIT of RAM_ERASE_7: label is "00000000";
attribute INIT of RAM_ERASE_8: label is "00000000";
--
-- Placement of the 8 RAM_ERASE (Optional)
--attribute RLOC: string;
--attribute RLOC of RAM_ERASE_0: label is "R0C0.S1";
--attribute RLOC of RAM_ERASE_1: label is "R0C0.S1";
--attribute RLOC of RAM_ERASE_2: label is "R1C0.S1";
--attribute RLOC of RAM_ERASE_3: label is "R1C0.S1";
--attribute RLOC of RAM_ERASE_4: label is "R2C0.S1";
--attribute RLOC of RAM_ERASE_5: label is "R2C0.S1";
--attribute RLOC of RAM_ERASE_6: label is "R3C0.S1";
--attribute RLOC of RAM_ERASE_7: label is "R3C0.S1";
--
-- Signal Declarations:
--
-- signal VCC : std_logic;
-- signal GND : std_logic;
--
begin
-- VCC <= '1';
-- GND <= '0';

-- SelectRAM instantiation = 9 x RAM32x1s
-- Generate not used because no Initialization by attribute
--RAM_32x1 : for i in 0 to 8 
--generate RAM_ERASE : RAM32x1s_1  
--	port map (
--	WE => WRITE_RAM,
--	WCLK => CLK,
--	D => DATA_IN(i),
--	A0 => ADDR(0),
--	A1 => ADDR(1),
--	A2 => ADDR(2),
--	A3 => ADDR(3),
--      A4 => ADDR(4),
--      A5 => ADDR(5),
--	O => DATA_WRITE(i)
--	);
--end generate;
RAM_ERASE_0 : RAM32x1s_1 
	port map (
	WE => WRITE_RAM,
	WCLK => CLK,
	D => DATA_IN(0),
	A0 => ADDR(0),
	A1 => ADDR(1),
	A2 => ADDR(2),
	A3 => ADDR(3),
        A4 => ADDR(4),
	O => DATA_WRITE(0)
	);
--
RAM_ERASE_1 : RAM32x1s_1  
	port map (
	WE => WRITE_RAM,
	WCLK => CLK,
	D => DATA_IN(1),
	A0 => ADDR(0),
	A1 => ADDR(1),
	A2 => ADDR(2),
	A3 => ADDR(3),
   A4 => ADDR(4),
	O => DATA_WRITE(1)
	);
--
RAM_ERASE_2 : RAM32x1s_1  
	port map (
	WE => WRITE_RAM,
	WCLK => CLK,
	D => DATA_IN(2),
	A0 => ADDR(0),
	A1 => ADDR(1),
	A2 => ADDR(2),
	A3 => ADDR(3),
   A4 => ADDR(4),
	O => DATA_WRITE(2)
	);
--
RAM_ERASE_3 : RAM32x1s_1  
	port map (
	WE => WRITE_RAM,
	WCLK => CLK,
	D => DATA_IN(3),
	A0 => ADDR(0),
	A1 => ADDR(1),
	A2 => ADDR(2),
	A3 => ADDR(3),
   A4 => ADDR(4),
	O => DATA_WRITE(3)
	);
--
RAM_ERASE_4 : RAM32x1s_1  
	port map (
	WE => WRITE_RAM,
	WCLK => CLK,
	D => DATA_IN(4),
	A0 => ADDR(0),
	A1 => ADDR(1),
	A2 => ADDR(2),
	A3 => ADDR(3),
        A4 => ADDR(4),
	O => DATA_WRITE(4)
	);
--
RAM_ERASE_5 : RAM32x1s_1  
	port map (
	WE => WRITE_RAM,
	WCLK => CLK,
	D => DATA_IN(5),
	A0 => ADDR(0),
	A1 => ADDR(1),
	A2 => ADDR(2),
	A3 => ADDR(3),
        A4 => ADDR(4),
	O => DATA_WRITE(5)
	);
--
RAM_ERASE_6 : RAM32x1s_1  
	port map (
	WE => WRITE_RAM,
	WCLK => CLK,
	D => DATA_IN(6),
	A0 => ADDR(0),
	A1 => ADDR(1),
	A2 => ADDR(2),
	A3 => ADDR(3),
        A4 => ADDR(4),
	O => DATA_WRITE(6)
	);
--
RAM_ERASE_7 : RAM32x1s_1  
	port map (
	WE => WRITE_RAM,
	WCLK => CLK,
	D => DATA_IN(7),
	A0 => ADDR(0),
	A1 => ADDR(1),
	A2 => ADDR(2),
	A3 => ADDR(3),
        A4 => ADDR(4),
	O => DATA_WRITE(7)
	);
--
RAM_ERASE_8 : RAM32x1s_1  
	port map (
	WE => WRITE_RAM,
	WCLK => CLK,
	D => DATA_IN(8),
	A0 => ADDR(0),
	A1 => ADDR(1),
	A2 => ADDR(2),
	A3 => ADDR(3),
        A4 => ADDR(4),
	O => DATA_WRITE(8)
	);
	--
end INIT_9_RAM32x1s_arch;
